use crate::cmd::config::app_config;
use crate::ems::bms::gold::gold_bms::GoldBms;
use crate::ems::emu::device::{DevType, Device};
use crate::ems::emu::BroadcastMessage;
use crate::ems::pcs::pcs::Pcs;
use crate::internal::utils;
use crate::internal::utils::mqtt::mqtt_client;
use log::{error, info};
use mosquitto_rs::{Event, Message, QoS};
use regex::Regex;
use std::collections::HashMap;
use std::sync::Arc;
use tokio::sync::broadcast;
use tokio::task::JoinHandle;
use tokio::{join, sync};

/// EMU 能量管理处理单元
/// devices: 设备集合
/// tx: 广播发生器
pub struct Emu {
    pub devices: Arc<HashMap<DevType, Arc<Device>>>,
    pub tx: broadcast::Sender<BroadcastMessage>,
}

impl Emu {
    /// 创建EMU实例
    pub async fn new() -> Self {
        //初始化日志
        utils::log::init_log("inpower_iot_mgc_rs::ems::emu::*", "emu/emu.log").await;
        //EMU消息广播器
        let (tx, _) = broadcast::channel::<BroadcastMessage>(8);
        let mut devices = HashMap::new();
        {
            //↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓ 创建PCS
            let (pcs_tx, pcs_rx) = sync::mpsc::channel::<String>(8);
            let pcs = Pcs::new(pcs_tx, tx.subscribe());
            //↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑

            //↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓ 创建BMS
            let (bms_tx, bms_rx) = sync::mpsc::channel::<String>(8);
            let bms = GoldBms::new(bms_tx, tx.subscribe());
            //↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑
            let (pcs, bms) = join!(pcs, bms);

            if let Ok(b) = bms {
                devices.insert(
                    DevType::BMS,
                    Device::new(b.id.clone(), "BMS", Arc::new(b), bms_rx),
                );
            }

            // if let Ok(p) = pcs {
            //     devices.insert(
            //         DevType::PCS,
            //         Device::new(p.id.clone(), "PCS", Arc::new(p), pcs_rx),
            //     );
            // }
        }
        info!("EMU初始化完成");
        Emu {
            devices: Arc::new(devices),
            tx,
        }
    }

    pub async fn run(self: Arc<Self>) {
        let mut handles = Vec::new();
        match self.mqtt().await {
            Ok(h) => handles.push(h),
            Err(e) => error!("MQTT 启动失败: {}", e),
        }
        for dev in self.devices.values() {
            let device_clone = Arc::clone(dev);
            handles.push(tokio::spawn(async move {
                device_clone.start().await;
            }))
        }
        // let x = &self.devices;
        // let a = x.get(&"pcs".to_string()).unwrap().clone();
        // tokio::spawn(async move {
        //     loop {
        //         let str = a.channel.lock().await.recv().await.unwrap();
        //         info!("[PCS] {}", str)
        //     }
        // });
        // let b = x.get(&"bms".to_string()).unwrap().clone();
        // tokio::spawn(async move {
        //     loop {
        //         let str = b.channel.lock().await.recv().await.unwrap();
        //         info!("[BMS] {}", str)
        //     }
        // });
        // let sender = self.tx.clone();
        // tokio::spawn(async move {
        //     loop {
        //         sender.send(String::from("BMS发出了信号")).unwrap();
        //         tokio::time::sleep(tokio::time::Duration::from_secs(5)).await;
        //     }
        // });
        for handler in handles {
            match handler.await {
                Ok(_) => {}
                Err(e) => {
                    log::error!("设备运行错误: {}", e.to_string());
                    continue;
                }
            }
        }
    }

    async fn mqtt(&self) -> anyhow::Result<JoinHandle<()>> {
        //mqtt Client
        let client = &mqtt_client().await?.mosq;
        let subscriptions = client.subscriber().unwrap();
        let topic_vec = &app_config().mqtt.topic;
        //订阅这些主题
        for topic in topic_vec.iter() {
            client.subscribe(topic, QoS::AtMostOnce).await?;
        }
        let sender = self.tx.clone();
        let devices = Arc::clone(&self.devices);
        //异步处理mqtt推送的消息
        Ok(tokio::spawn(async move {
            loop {
                if let Ok(message) = subscriptions.recv().await {
                    match message {
                        Event::Message(msg) => {
                            if let Err(e) = mqtt_msg_handler(msg, &sender, &devices).await {
                                error!("[mqtt] 订阅消息分发失败: {}", e.to_string());
                            }
                        }
                        Event::Connected(_) => {
                            info!("[mqtt] 已连接")
                        }
                        Event::Disconnected(_) => {
                            info!("[mqtt] 断开连接")
                        }
                    }
                }
            }
        }))
    }
}

async fn mqtt_msg_handler(
    msg: Message,
    sender: &broadcast::Sender<BroadcastMessage>,
    devices: &Arc<HashMap<DevType, Arc<Device>>>,
) -> anyhow::Result<()> {
    use serde_json::Value;
    let json = String::from_utf8(msg.payload)?;
    let v: Value = serde_json::from_str(json.as_str())?;
    //{"key":"open_pcs","value":12}
    if let (Some(k), Some(v)) = (v.get("key"), v.get("value")) {
        //key must string, value must number
        if k.is_string() && v.is_number() {
            if let (Some(key), Some(value)) = (k.as_str(), v.as_number()) {
                //topic 必须匹配 /asw/${dev}/${index}/setyt
                if let Some((dev, _index)) = extract_with_regex(msg.topic.as_str()) {
                    //广播给设备带上dev_id
                    let id = match dev {
                        "bms" => {
                            let bms = devices
                                .get(&DevType::BMS)
                                .ok_or(anyhow::anyhow!("设备容器中找不到BMS"))?;
                            Some(bms.id.as_str())
                        }
                        "pcs" => {
                            let pcs = devices
                                .get(&DevType::PCS)
                                .ok_or(anyhow::anyhow!("设备容器中找不到PCS"))?;
                            Some(pcs.id.as_str())
                        }
                        other => {
                            error!("[mqtt] 目前没有<{}>这个设备", other);
                            None
                        }
                    };
                    if let Some(id) = id {
                        sender.send(BroadcastMessage {
                            id: id.to_string(),
                            key: key.to_string(),
                            value: Arc::new(value.clone()),
                        })?;
                    }
                } else {
                    error!("[mqtt] topic解析错误: {}", msg.topic)
                }
            }
        } else {
            error!("[mqtt] JSON解析错误, key is {}, value is {}", k, v)
        }
    } else {
        error!("[mqtt] JSON解析错误")
    }
    Ok(())
}

fn extract_with_regex(path: &str) -> Option<(&str, &str)> {
    let re = Regex::new(r"^/asw/([^/]+)/([^/]+)/setyt$").unwrap();
    if let Some(caps) = re.captures(path) {
        Some((caps.get(1)?.as_str(), caps.get(2)?.as_str()))
    } else {
        None
    }
}
