use crate::internal::modbus::{BaseModbusCode, CsvData};
use anyhow::anyhow;
use std::collections::HashMap;

/// 寄存器类型
#[derive(Debug, Eq, PartialEq)]
pub enum RegisterType {
    //线圈寄存器 0
    Coil,
    //离散输入寄存器 1
    DiscreteInput,
    //保持寄存器 2
    HoldingRegister,
    //输入寄存器 3
    InputRegister,
}

impl TryFrom<u8> for RegisterType {
    type Error = anyhow::Error;

    fn try_from(value: u8) -> Result<Self, Self::Error> {
        match value {
            0 => Ok(RegisterType::Coil),
            1 => Ok(RegisterType::DiscreteInput),
            2 => Ok(RegisterType::HoldingRegister),
            3 => Ok(RegisterType::InputRegister),
            _ => anyhow::bail!("无效的寄存器类型！"),
        }
    }
}

/// 数据类型
#[derive(Debug)]
pub enum ModbusDataType {
    Bit,
    U16,
    I16,
    U32,
    I32,
    F32,
    F64,
}

impl TryFrom<String> for ModbusDataType {
    type Error = anyhow::Error;

    fn try_from(value: String) -> Result<Self, Self::Error> {
        match value.as_str() {
            "bit" => Ok(ModbusDataType::Bit),
            "u16" => Ok(ModbusDataType::U16),
            "i16" => Ok(ModbusDataType::I16),
            "u32" => Ok(ModbusDataType::U32),
            "i32" => Ok(ModbusDataType::I32),
            "f32" => Ok(ModbusDataType::F32),
            "f64" => Ok(ModbusDataType::F64),
            _ => anyhow::bail!("无效的数据类型！"),
        }
    }
}

#[derive(Debug)]
pub enum Rw {
    R,
    W,
    RW,
}

impl TryFrom<String> for Rw {
    type Error = anyhow::Error;

    fn try_from(value: String) -> Result<Self, Self::Error> {
        match value.as_str() {
            "r" => Ok(Rw::R),
            "w" => Ok(Rw::W),
            "rw" => Ok(Rw::RW),
            _ => anyhow::bail!("无效的读写！"),
        }
    }
}

#[derive(Debug)]
pub struct ModbusCode {
    pub addr: u16,
    pub code: RegisterType,
    pub data: Option<[u8; 2]>,
    pub name: String,
    pub data_type: Option<ModbusDataType>,
    pub factor: Option<f32>,
    pub rw: Option<Rw>,
    pub unit: Option<String>,
    pub note: Option<HashMap<i32, String>>,
    pub desc: Option<String>,
}

impl ModbusCode {}

impl BaseModbusCode for ModbusCode {
    fn addr(&self) -> u16 {
        self.addr
    }
}

impl BaseModbusCode for &mut ModbusCode {
    fn addr(&self) -> u16 {
        self.addr
    }
}

impl TryFrom<CsvData> for ModbusCode {
    type Error = anyhow::Error;

    fn try_from(value: CsvData) -> Result<Self, Self::Error> {
        let addr = value.address.ok_or(anyhow!("寄存器地址为空！"))?;
        let code = value.code.ok_or(anyhow!("寄存器类型为空！"))?.try_into()?;
        let name = value.name.ok_or(anyhow!("寄存器名称为空！"))?;
        let data_type = match value.t {
            None => None,
            Some(it) => Some(it.try_into()?),
        };
        let rw = match value.rw {
            None => None,
            Some(it) => Some(it.try_into()?),
        };
        let factor = value.factor;
        let unit = value.unit;
        let desc = value.desc;
        let mut note = None;
        if let Some(n) = value.note {
            let mut hash_map = HashMap::new();
            for x in n.split('\n').into_iter() {
                let kv = x.split(':').collect::<Vec<&str>>();
                if let Some(key) = kv.get(0) {
                    let k = if key.starts_with("0x") {
                        i32::from_str_radix(key.trim().replace("0x", "").as_ref(), 16)?
                    } else {
                        key.parse::<i32>()?
                    };
                    if let Some(val) = kv.get(1) {
                        hash_map.insert(k, val.trim().to_string());
                    }
                }
            }
            note = Some(hash_map);
        }
        Ok(ModbusCode {
            addr,
            code,
            data: None,
            name,
            data_type,
            factor,
            rw,
            unit,
            note,
            desc,
        })
    }
}
